@@ -146,6 +146,23 @@ fn redirect_post() {
     assert_eq!(resp.header("x-foo").unwrap(), "bar");
 }
 
+#[test]
+fn redirect_post_with_data() {
+    test::set_handler("/redirect_post_d1", |unit| {
+        assert_eq!(unit.header("Content-Length").unwrap(), "4");
+        test::make_response(302, "Go here", vec!["Location: /redirect_post_d2"], vec![])
+    });
+    test::set_handler("/redirect_post_d2", |unit| {
+        assert_eq!(unit.header("Content-Length"), None);
+        assert_eq!(unit.method, "GET");
+        test::make_response(200, "OK", vec![], vec![])
+    });
+    let resp = post("test://host/redirect_post_d1")
+        .send_string("data")
+        .unwrap();
+    assert_eq!(resp.status(), 200);
+}
+
 #[test]
 fn redirect_308() {
     test::set_handler("/redirect_get3", |_| {
